;*******************************************************************************


;    Fechadura Eletrnica Biomtrica para Acesso a Ambiente Privados

;    Para Compilador Assembly MPLAB-IDE V6.61

;    (Arquivo Asm adaptado da compilao do Great Cow BASIC (0.98.06 2019-06-12 
;    (Windows 32 bit)))

;    C/ PIC16f628a e mdulo sensor biomtrico DY50

;    Autor: Claudio Lrios

;    Data: 28/05/2020

;    Microcontrolador: PIC 16f628a



;    Objetivo: Acesso a ambientes privados atravs do uso da impresso digital

;    Cadastrar digital do Administrador do Sistema:
;    Comece cadastrando a digital da pessoa que ser o administrador.
;    Aperte o interruptor 'sw_prog'. O administrador cadastrar o dedo que ser
;    usado  para permitir gravar outras digitais no sistema.
;    Ao apagar o led finger, coloque de novo o mesmo dedo. Quando piscar este led
;    indicar que houve sucesso na gravao.

;    Cadastrar outras digitais:
;    O administrador usa o dedo cadastrado para iniciar o processo. Assim que co-
;    locar o dedo dever acender o led prog. Pea para a outra pessoa colocar o
;    dedo que usar para o acesso. Ao apagar o led finger pea que coloque de
;    novo o mesmo dedo.
;    Quando piscar este led vrias vezes indicar sucesso na gravao.
;    Repita os mesmos passos com cada pessoa que ter acesso ao ambiente.

;    Deletar todas as digitais:
;    Desligue a alimentao do circuito. Aperte o boto sw_del_lib e mantenha
;    apertado. Ligue a alimentao. Quando led prog piscar varias vezes, solte
;    o boto sw_del_lib. Recadastre novamente as digitais conforme os passos
;    acima.

;    Ateno: use um adaptador de tenso de 3v3 para 5 volts na sada do sensor
;    nomeada de RX ( com 3v no funciona a uart a 57600 bauds). O mesmo para TX.

;    Ateno: Para compilar sem dar erro, voc deve ajustar as opoes de
;    montagem do arquivo no MPLAB conforme abaixo:
;    Clique na barra de ferramentas do MPLAB-IDE em:
;      Projects > Build Options...> fb_v3.asm e  marque a caixa :
;      'Disable Case Sensitivity' e de 'ok'.

;    Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
;    Proposta didtica apenas, sendo testada somente em placa de protoboard.
;********************************************************************************



 LIST p=16F628A, r=DEC
#include <P16F628A.inc>
     __CONFIG _CP_OFF & _LVP_OFF & _MCLRE_OFF & _WDT_ON & _HS_OSC 
     ERRORLEVEL 1, -302 ;desativa alerta de verificao de banco
;********************************************************************************
;Defines do sensor DY50
;#define FP_OK 0x00
;#define FP_PACKETRECIEVEERR 0x01
;#define FP_NOFINGER 0x02
;#define FP_IMAGEFAIL 0x03
;#define FP_IMAGEMESS 0x06
;#define FP_FEATUREFAIL 0x07
;#define FP_NOMATCH 0x08
;#define FP_NOTFOUND 0x09
;#define FP_ENROLLMISMATCH 0x0A
;#define FP_BADLOCATION 0x0B
;#define FP_DBRANGEFAIL 0x0C
;#define FP_UPLOADFEATUREFAIL 0x0D
;#define FP_PACKETRESPONSEFAIL 0x0E
;#define FP_UPLOADFAIL 0x0F
;#define FP_DELETEFAIL 0x10
;#define FP_DBCLEARFAIL 0x11
;#define FP_PASSFAIL 0x13
;#define FP_INVALIDIMAGE 0x15
;#define FP_FLASHERR 0x18
;#define FP_INVALIDREG 0x1A
;#define FP_ADDRCODE 0x20
;#define FP_PASSVERIFY 0x21
;#define FP_STARTCODE 0xEF01
;#define FP_COMMANDPACKET 0x1
;#define FP_DATAPACKET 0x2
;#define FP_ACKPACKET 0x7
;#define FP_ENDDATAPACKET 0x8
;#define FP_TIMEOUT 0xFF
;#define FP_BADPACKET 0xFE
;#define FP_GETIMAGE 0x01
;#define FP_IMAGE2TZ 0x02
;#define FP_REGMODEL 0x05
;#define FP_STORE 0x06
;#define FP_LOAD 0x07
;#define FP_UPLOAD 0x08
;#define FP_DELETE 0x0C
;#define FP_EMPTY 0x0D
;#define FP_SETPASSWORD 0x12
;#define FP_VERIFYPASSWORD 0x13
;#define FP_HISPEEDSEARCH 0x1B
;#define FP_TEMPLATECOUNT 0x1D
;----------------------------------
#define NUM_MAX_FP 163  ;Numero mximo suportado de digitais
#define TEMPO_FECHADURA_ACIONADA 2 ;' em segundos
#define TEMPO_MAX 180 ;tempo mximo em modo gravao (aprox. 40 segundos)

;----- Nota sobre a comunicao da serial-----------------------
;( Definio da USART foi feito no cdigo com asm intercalado)
;#define USART_BAUD_RATE 57600
;#define USART_BLOCKING true
;Configura pinos de entradas e sadas
;#define SerInPort PORTB.1  'pino 7 RX
;Dir SerInPort In 'in


;Variavis
AUX	EQU	32
DATAR	EQU	322  ;buffer de recepo da serial
DELAYTEMP	EQU	112
DELAYTEMP2	EQU	113
GET_ACKPACK	EQU	33
IDLIDO	EQU	34
LOCATION	EQU	35
LOOPCOUNTER	EQU	36
NUMBUF	EQU	37
NUMID	EQU	38
POS_GRAVAR	EQU	39
QUANT	EQU	40
RESP	EQU	41
SUM	EQU	42
SYSBYTETEMPA	EQU	117
SYSBYTETEMPB	EQU	121
SYSBYTETEMPX	EQU	112
SYSTEMP1	EQU	43
SYSTEMP2	EQU	44
SYSTEMP3	EQU	45
SYSWAITTEMPMS	EQU	114
SYSWAITTEMPMS_H	EQU	115
SYSWAITTEMPS	EQU	116
SYSWAITTEMPUS	EQU	117
SYSWAITTEMPUS_H	EQU	118
TEMP	EQU	46  ; temporrio
VALOR	EQU	47  ; temporrio

;********************************************************************************

;Alias variables
EEADDRESS	EQU	155
EEDATAVALUE	EQU	154
SYSDATA_11	EQU	333
SYSDATA_9	EQU	331

;********************************************************************************

;Vectors
	ORG	0
	goto	Inicio_Programa
	ORG	4
	retfie

;********************************************************************************

;Start of program memory page 0
	ORG	5
Inicio_Programa
;Call initialisation routines
	call	INITSYS
;Start of the main program
	banksel	TRISB
	bsf	TRISB,1
;#define SerOutPort PORTB.2 'pino 8 TX
;Dir SerOutPort Out'out
	bcf	TRISB,2
;#define sw_del_lib PORTB.5  'pino 11 deletar todas digitais da flash
;Dir sw_del_lib In
	bsf	TRISB,5
;#define sw_prog    PORTA.4  'pino 3 - programar digital do ADMINISTRADOR
;Dir sw_prog In
	bsf	TRISA,4
;#define led_finger PORTB.4  'pino 10 - indica para colocar/retirar o dedo
;Dir led_finger Out
	bcf	TRISB,4
;#define led_erro   PORTA.2  'pino 1 - led indicador de algum erro
;Dir led_erro Out
	bcf	TRISA,2
;#define led_acesso_ok PORTB.0  'pino 6 - sada para acionar fechadura
;Dir led_acesso_ok Out
	bcf	TRISB,0
;#define led_prog PORTA.0  'pino 17 - aceso= modo prog
;Dir led_prog Out
	bcf	TRISA,0
;#define led_com_ok PORTA.1  'pino 18 -led programao realizado com sucesso
;Dir led_com_ok Out
	bcf	TRISA,1
;#define led_teste PORTA.3  'pino 2 -led teste
;Dir led_teste Out
	bcf	TRISA,3

	banksel	NUMID
	clrf	NUMID
;----- Main body of program commences here.
;Usart definies
;SPBRG = 21 'xtal 20mhz 57600 bauds  brgh=1
	movlw	21
	banksel	SPBRG
	movwf	SPBRG
;RCSTA = 0b10010000 'assyncronous rx config pino 7
	movlw	144
	banksel	RCSTA
	movwf	RCSTA
;TXSTA = 0b10100100 'assyncronous tx config  pino 8
	movlw	164
	banksel	TXSTA
	movwf	TXSTA
;Watchdog definies
;OPTION_REG = 0B10001111 'WTD com tempo de 2,3 seg
	movlw	143
	movwf	OPTION_REG
;epread 0 , aux 'le eeprom end 0
	clrf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUX
	movwf	AUX
;if aux = 0xff then
	incf	AUX,W
	btfss	STATUS, Z
	goto	ENDIF1
;epwrite 0,0 'reseta end 0 no ps programao do firmware
	banksel	EEADDRESS
	clrf	EEADDRESS
	clrf	EEDATAVALUE
	banksel	STATUS
	call	EPWRITE
;end if
ENDIF1
;epread 0 , numID 'retorna id da flash do pic
	banksel	EEADDRESS
	clrf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	NUMID
	movwf	NUMID
;led_com_ok = 1
	bsf	PORTA,1
;delay_seg(1) 'Tempo de boot-up do sensor
	movlw	1
	movwf	VALOR
	call	DELAY_SEG
;led_com_ok = 0
	bcf	PORTA,1
;CMD_Verify_PassWord 'envia comando senha do sensor
	call	CMD_VERIFY_PASSWORD
;resp = Get_AckPack  'obtm a resposta  //erro no funciona a recepo
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if  resp <> FP_OK  then
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF2
;pisca_led_erro 'algum erro ocorreu, necessita reset
	call	PISCA_LED_ERRO
;do forever 'aguarda wtd timer fazer reset (ou manual)
SysDoLoop_S1
;Loop
	goto	SysDoLoop_S1
SysDoLoop_E1
;end if
ENDIF2
;del_all_flash 'testa se deve deletar toda as impresses digitais da flash
	call	DEL_ALL_FLASH
;----------------lao de repetio  principal--------------------------
;Do
SysDoLoop_S2
;CLRWDT ' restart WTD;
	clrwdt
;BOTO DE GRAVAR ADM
;if sw_prog = 0 then
	btfsc	PORTA,4
	goto	ENDIF3
;led_prog = 1 'acende led de programao
	bsf	PORTA,0
;do while sw_prog = 0
SysDoLoop_S3
	btfsc	PORTA,4
	goto	SysDoLoop_E3
;delay_100ms 'aguarda soltar boto sw_prog
	call	DELAY_100MS
;loop
	goto	SysDoLoop_S3
SysDoLoop_E3
;end if
ENDIF3
;if led_prog = 1 then
	btfss	PORTA,0
	goto	ENDIF4
;gravar_digital(0) 'grava digital do administrador
	clrf	POS_GRAVAR
	call	GRAVAR_DIGITAL
;led_prog = 0 'sai do modo de programao da digital do adm
	bcf	PORTA,0
;end if
ENDIF4
;procurar_digital 'verifica se existe digital a ser comparada
	call	PROCURAR_DIGITAL
;delay_200ms
;Loop
	goto	SysDoLoop_S2
SysDoLoop_E2
;------------------fim da rotina principal-----------------------------

BASPROGRAMEND
	sleep
	goto	BASPROGRAMEND

;********************************************************************************
;/*******************************************************************************
;      Rotina  de Pesquisa por digital igual a uma j gravada na  Memria Flash
;      obs. Fixado no Buffer 1 e nas paginas 0x0 ate 0xA3
;*******************************************************************************/
CMD_FASTSEARCH
;Header 'envia header em modo comando
	call	HEADER
;txreg = 0x00 'lenght msb
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop39
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop39
;txreg = 0x08 'lenght lsb
	movlw	8
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop40
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop40
;txreg = FP_HISPEEDSEARCH '0x1B comando
	movlw	27
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop41
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop41
;txreg = 0x01 'buffer ID escolhido para comparao
	movlw	1
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop42
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop42
;txreg = 0x00 'pag inicial msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop43
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop43
;txreg = 0x00 'pag inicial lsb
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop44
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop44
;txreg = 0x00 'pag final msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop45
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop45
;txreg=0xA3 'pag final lsb
	movlw	163
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop46
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop46
;txreg=0x00 'sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop47
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop47
;txreg=0xc8 'sum lsb
	movlw	200
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop48
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop48
	banksel	STATUS
	return


;*******************************************************************************
;      Rotina  de Criao de Modlo e Armazenamento em Buffer
;      ID= 1 ou 2 (apenas 2 buffers)
;*******************************************************************************/

CMD_FP_CREATEMODEL
;Header 'envia header em modo comando
	call	HEADER
;txreg=0x00 'lenght 2 bytes
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop26
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop26
;txreg=0x03;
	movlw	3
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop27
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop27
;txreg=FP_REGMODEL '0x05;//comando
	movlw	5
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop28
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop28
;txreg=0x00 'sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop29
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop29
;txreg=0x09 'sum lsb
	movlw	9
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop30
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop30
	banksel	STATUS
	return

;*******************************************************************************
;      Rotina  para Deletar todas as Digitais Armazenadas na Flash
;*******************************************************************************/

CMD_FP_EMPTY
;Header 'envia header em modo comando
	call	HEADER
;txreg=0x00 'lenght msb
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop49
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop49
;txreg=0x03 'lenght lsb
	movlw	3
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop50
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop50
;txreg=FP_EMPTY '0x0d comando
	movlw	13
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop51
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop51
;txreg=0x00 'sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop52
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop52
;txreg=0x11 'sum lsb
	movlw	17
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop53
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop53
	banksel	STATUS
	return

;*******************************************************************************
;      Rotina  de Gerao de Arquivo e Armazenamento em Buffer
;      ID= 1 ou 2 (apenas 2 buffers)
;*******************************************************************************/
CMD_FP_IMAGE2TZ
;if numbuf <> 1 then
	decf	NUMBUF,W
	btfsc	STATUS, Z
	goto	ENDIF27
;numbuf=2;
	movlw	2
	movwf	NUMBUF
;end if
ENDIF27
;Header 'envia header em modo comando
	call	HEADER
;txreg=0x00;//lenght msb
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop20
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop20
;txreg=0x04; //lenght lsb
	movlw	4
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop21
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop21
;txreg=FP_IMAGE2TZ;//0x02;//comando
	movlw	2
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop22
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop22
;txreg=numbuf;
	banksel	NUMBUF
	movf	NUMBUF,W
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop23
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop23
;txreg=0x00;//sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop24
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop24
;if numbuf = 1 then 'ID=1
	banksel	NUMBUF
	decf	NUMBUF,W
	btfss	STATUS, Z
	goto	ELSE28_1
;txreg=0x08 'sum lsb
	movlw	8
	movwf	TXREG
;else 'ID=2
	goto	ENDIF28
ELSE28_1
;txreg=0x09 'sum lsb
	movlw	9
	movwf	TXREG
;end if
ENDIF28
;wait until trmt=1 '
SysWaitLoop25
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop25
	banksel	STATUS
	return

;*******************************************************************************
;      Rotina  de obteno da digital - CMD_Get_Image
;*******************************************************************************/

CMD_GETIMAGE
;Header 'envia header em modo comando
	call	HEADER
;txreg=0x00 'lenght msb
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop15
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop15
;txreg=0x03 ;lenght lsb
	movlw	3
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop16
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop16
;txreg=FP_GETIMAGE  '0x01;//comando
	movlw	1
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop17
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop17
;txreg=0x00 'sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop18
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop18
;txreg=0x05 'sum lsb
	movlw	5
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop19
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop19
	banksel	STATUS
	return

;*******************************************************************************
;      Rotina  de Armazenamento de Arquivo na Memria Flash
;      ID= 1 ou 2 (apenas 2 buffers)
;*******************************************************************************/
CMD_STOREMODEL
;Dim sum as byte : sum = 14;
	movlw	14
	movwf	SUM
;sum = sum + location 'soma ao sum
	movf	LOCATION,W
	addwf	SUM,F
;Header 'envia header em modo comando
	call	HEADER
;txreg = 0x00 'lenght msb
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop31
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop31
;txreg = 0x06 'lenght lsb
	movlw	6
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop32
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop32
;txreg = FP_STORE '0x06;//comando
	movlw	6
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop33
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop33
;txreg = 0x01 'aponta para  o modelo no buffer 1
	movlw	1
	banksel	TXREG
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop34
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop34
;txreg = 0x00 'id msb (max 160 imagens)
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop35
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop35
;txreg = location 'id lsb
	banksel	LOCATION
	movf	LOCATION,W
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop36
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop36
;txreg = 0x00 'sum msb
	banksel	TXREG
	clrf	TXREG
;wait until trmt = 1 '
SysWaitLoop37
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop37
;txreg = sum 'sum lsb
	banksel	SUM
	movf	SUM,W
	movwf	TXREG
;wait until trmt = 1 '
SysWaitLoop38
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop38
	banksel	STATUS
	return
;*******************************************************************************
;      Rotina de Envio da Instruo 'CMD_VerifyPassWord'
;      0x00000000 = envia 'Password Default'
;  obs. Para usar outra 'password' necessitar alterar a rotina abaixo.
;*******************************************************************************/

CMD_VERIFY_PASSWORD
;Header 'envia header em modo comando
	call	HEADER
;txreg=0x00;//lenght 2 bytes
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop1
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop1
;txreg=0x07;
	movlw	7
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop2
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop2
;txreg=FP_VERIFYPASSWORD;//comando 0x13
	movlw	19
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop3
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop3
;for LoopCounter = 0 to 4'envia senha 0x00000000 e msb sum=0x00
	movlw	255
	banksel	LOOPCOUNTER
	movwf	LOOPCOUNTER
SysForLoop3
	incf	LOOPCOUNTER,F
;txreg=0x00;//senha 4 bytes
	clrf	TXREG
;wait until trmt=1 '
SysWaitLoop4
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop4
;next
	movlw	4
	banksel	LOOPCOUNTER
	subwf	LOOPCOUNTER,W
	btfss	STATUS, C
	goto	SysForLoop3
SysForLoopEnd3
;txreg=0x1b;//sum lsb
	movlw	27
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop5
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop5
	banksel	STATUS
	return

;==============================================================================
;                      Rotinas de Delay
;==============================================================================*/

DELAY_100MS
;CLRWDT
	clrwdt
;wait 100 Ms  '100 milissegundos
	movlw	100
	movwf	SysWaitTempMS
	clrf	SysWaitTempMS_H
	goto	Delay_MS

;********************************************************************************

DELAY_200MS
;CLRWDT
	clrwdt
;wait 200 Ms
	movlw	200
	movwf	SysWaitTempMS
	clrf	SysWaitTempMS_H
	goto	Delay_MS

;********************************************************************************

DELAY_200US
;CLRWDT
	clrwdt
;wait 200 us
	movlw	1
	movwf	DELAYTEMP2
DelayUSO1
	clrf	DELAYTEMP
DelayUS1
	decfsz	DELAYTEMP,F
	goto	DelayUS1
	decfsz	DELAYTEMP2,F
	goto	DelayUSO1
	movlw	76
	movwf	DELAYTEMP
DelayUS2
	decfsz	DELAYTEMP,F
	goto	DelayUS2
	return

;********************************************************************************

DELAY_SEG
;for LoopCounter = 0 to valor
	movlw	255
	movwf	LOOPCOUNTER
	movlw	0
	subwf	VALOR,W
	btfss	STATUS, C
	goto	SysForLoopEnd1
SysForLoop1
	incf	LOOPCOUNTER,F
;CLRWDT
	clrwdt
;wait 1 s
	movlw	1
	movwf	SysWaitTempS
	call	Delay_S
;next
	movf	VALOR,W
	subwf	LOOPCOUNTER,W
	btfss	STATUS, C
	goto	SysForLoop1
SysForLoopEnd1
	return

;*******************************************************************************
;      Rotina de Apagamento de todas as digitais  na Flash
;      Obs. Deve ser executado somente no setup (apertando boto sw_del_lib
;      e religando a alimentao)
;*******************************************************************************/


DEL_ALL_FLASH
;if sw_del_lib = 0 then 'testa boto de apagamento de todas as digitais gravadas
	btfsc	PORTB,5
	goto	ENDIF23
;led_com_ok = 1
	bsf	PORTA,1
;delay_seg(5) 'delay de 5 segundos
	movlw	5
	movwf	VALOR
	call	DELAY_SEG
;if sw_del_lib = 0 then
	btfsc	PORTB,5
	goto	ENDIF24
;CMD_FP_Empty
	call	CMD_FP_EMPTY
;resp = Get_AckPack 'obtm a resposta
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK  then
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ELSE25_1
;pisca_led_erro 'erro no apagamento geral. No houve apagamento!
	call	PISCA_LED_ERRO
;else
	goto	ENDIF25
ELSE25_1
;se apagamento geral foi bem sucedido pisca_led prog 10 vezes
;numID = 0
	clrf	NUMID
;epwrite(0,numID) 'zera ponteiro
	banksel	EEADDRESS
	clrf	EEADDRESS
	banksel	NUMID
	movf	NUMID,W
	banksel	EEDATAVALUE
	movwf	EEDATAVALUE
	banksel	STATUS
	call	EPWRITE
;for LoopCounter = 1 to 10
	clrf	LOOPCOUNTER
SysForLoop5
	incf	LOOPCOUNTER,F
;led_prog = 1
	bsf	PORTA,0
;delay_100ms
	call	DELAY_100MS
;led_prog = 0
	bcf	PORTA,0
;delay_100ms
	call	DELAY_100MS
;next
	movlw	10
	subwf	LOOPCOUNTER,W
	btfss	STATUS, C
	goto	SysForLoop5
SysForLoopEnd5
;end if
ENDIF25
;end if
ENDIF24
;led_com_ok = 0
	bcf	PORTA,1
;do while sw_del_lib = 0 'aguarda soltar interruptor
SysDoLoop_S9
	btfsc	PORTB,5
	goto	SysDoLoop_E9
;delay_100ms
	call	DELAY_100MS
;Loop
	goto	SysDoLoop_S9
SysDoLoop_E9
;End if
ENDIF23
	return

;********************************************************************************
;   Rotina de Delay (Milissegundos)
;********************************************************************************

Delay_MS
	incf	SysWaitTempMS_H, F
DMS_START
	movlw	227
	movwf	DELAYTEMP2
DMS_OUTER
	movlw	6
	movwf	DELAYTEMP
DMS_INNER
	decfsz	DELAYTEMP, F
	goto	DMS_INNER
	decfsz	DELAYTEMP2, F
	goto	DMS_OUTER
	decfsz	SysWaitTempMS, F
	goto	DMS_START
	decfsz	SysWaitTempMS_H, F
	goto	DMS_START
	return

;********************************************************************************
;   Rotina de Delay (Segundos)
;********************************************************************************
Delay_S
DS_START
	movlw	232
	movwf	SysWaitTempMS
	movlw	3
	movwf	SysWaitTempMS_H
	call	Delay_MS
	decfsz	SysWaitTempS, F
	goto	DS_START
	return

;********************************************************************************

EPWRITE
;Variable alias
;Dim EEAddress Alias EEADR  'erv
;Dim EEDataValue Alias EEDATL_REF
;Disable interrupt
;IntOff
;Select data memory
;Start write
;SET WREN ON
	banksel	EECON1
	bsf	EECON1,WREN
;EECON2 = 0x55
	movlw	85
	movwf	EECON2
;EECON2 = 0xAA
	movlw	170
	movwf	EECON2
;SET WR ON
	bsf	EECON1,WR
;SET WREN OFF
	bcf	EECON1,WREN
;Wait for write to complete
;WAIT WHILE WR ON
SysWaitLoop7
	btfsc	EECON1,WR
	goto	SysWaitLoop7
;SET WREN OFF
	bcf	EECON1,WREN
;Restore interrupt
;IntOn
	banksel	STATUS
	return

;*******************************************************************************
;      Rotina  de Recepo dos dados do Sensor de Biomtrico (Acknowledge Packet)
;*******************************************************************************/

FN_GET_ACKPACK
;ateno: use um adaptador de tenso de 3v3 para 5 volts na sada do sensor
;nomeada de RX ( com 3v no funciona a uart a 57600 bauds).
;Dim  quant as byte;
;cren = 0 'apaga erros de recepo da uart
	bcf	RCSTA,CREN
;cren = 1
	bsf	RCSTA,CREN
;quant = rcreg 'limpa buffer da usart caso haja algum byte na FIFO
	movf	RCREG,W
	movwf	QUANT
;quant = 0
	clrf	QUANT
;led_com_ok=0
	bcf	PORTA,1
;Do until quant = 12
SysDoLoop_S4
	movlw	12
	subwf	QUANT,W
	btfsc	STATUS, Z
	goto	SysDoLoop_E4
;wait until rcif = 1 '
SysWaitLoop6
	btfss	PIR1,RCIF
	goto	SysWaitLoop6
;data(quant) = rcreg 'pega dado recebido
	movlw	low(DATAR)
	addwf	QUANT,W
	movwf	FSR
	bankisel	DATAR
	movf	RCREG,W
	movwf	INDF
;quant = quant + 1
	incf	QUANT,F
;led_com_ok = 1 'acende apenas na fase de recebimento de dados
	bsf	PORTA,1
;Loop
	goto	SysDoLoop_S4
SysDoLoop_E4
;led_com_ok = 0
	bcf	PORTA,1
;IDlido = data(11)
	banksel	SYSDATA_11
	movf	SYSDATA_11,W
	banksel	IDLIDO
	movwf	IDLIDO
;delay_200us
	call	DELAY_200US
;Get_AckPack = data(9) 'retorna valor pela funo
	banksel	SYSDATA_9
	movf	SYSDATA_9,W
	banksel	GET_ACKPACK
	movwf	GET_ACKPACK
	return

;*******************************************************************************
;      Rotina de Gravao de uma Digital  na Flash
;*******************************************************************************/
GRAVAR_DIGITAL
;dim temp as byte 'nmero de tentativas
;cren = 0
	bcf	RCSTA,CREN
;led_prog = 1 'acende led de programao
	bsf	PORTA,0
;led_finger = 1 'acende led 'coloque o dedo'
	bsf	PORTB,4
;cren = 1
	bsf	RCSTA,CREN
;1 captura da digital
;temp = TEMPO_MAX
	movlw   TEMPO_MAX
	movwf	TEMP
;do
SysDoLoop_S5
;delay_200ms'
	call	DELAY_200MS
;CMD_GetImage 'envia comando p/ fazer leitura de digital
	call	CMD_GETIMAGE
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;temp = temp - 1
	decf	TEMP,F
;Loop while ((resp <> FP_OK) and   (temp > 0 ))
	movf	resp,W
	movwf	SysBYTETempA
	clrf	SysBYTETempB
	call	SysCompEqual
	comf	SysByteTempX,F
	movf	SysByteTempX,W
	movwf	SysTemp1
	movf	temp,W
	movwf	SysBYTETempB
	clrf	SysBYTETempA
	call	SysCompLessThan
	movf	SysByteTempX,W
	movwf	SysTemp2
	movf	SysTemp1,W
	andwf	SysTemp2,W
	movwf	SysTemp3
	btfsc	SysTemp3,0
	goto	SysDoLoop_S5
SysDoLoop_E5
;if temp = 0 then
	movf	TEMP,F
	btfss	STATUS, Z
	goto	ENDIF15
;pisca_led_erro
	call	PISCA_LED_ERRO
;led_finger = 0
	bcf	PORTB,4
;Exit Sub 'sai da rotina por erro
	return
;End if
ENDIF15
;apaga led finger ('retire o dedo')
;aguarda a retirada do dedo
;led_finger = 0
	bcf	PORTB,4
;temp = TEMPO_MAX
	movlw	TEMPO_MAX
	movwf	TEMP
;do
SysDoLoop_S6
;delay_200ms 'delay
	call	DELAY_200MS
;CMD_GetImage 'envia comando p/ fazer leitura de digital
	call	CMD_GETIMAGE
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;temp = temp -1
	decf	TEMP,F
;Loop while ((resp <> FP_NOFINGER) and  (temp > 0 ))
	movf	resp,W
	movwf	SysBYTETempA
	movlw	2
	movwf	SysBYTETempB
	call	SysCompEqual
	comf	SysByteTempX,F
	movf	SysByteTempX,W
	movwf	SysTemp1
	movf	temp,W
	movwf	SysBYTETempB
	clrf	SysBYTETempA
	call	SysCompLessThan
	movf	SysByteTempX,W
	movwf	SysTemp2
	movf	SysTemp1,W
	andwf	SysTemp2,W
	movwf	SysTemp3
	btfsc	SysTemp3,0
	goto	SysDoLoop_S6
SysDoLoop_E6
;if temp = 0 then pisca_led_erro :  Exit Sub 'sai da rotina por erro
	movf	TEMP,F
	btfss	STATUS, Z
	goto	ENDIF16
	call	PISCA_LED_ERRO
	return
ENDIF16
;cria um modelo e coloca no buffer 1
;CMD_FP_Image2Tz(1) 'envia comando p/ fazer um modelo da digital
	movlw	1
	movwf	NUMBUF
	call	CMD_FP_IMAGE2TZ
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then pisca_led_erro : Exit Sub '/sai da rotina por erro
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF17
	call	PISCA_LED_ERRO
	return
ENDIF17
;2 captura da mesma digital
;led_finger = 1 'acende led 'coloque o dedo'
	bsf	PORTB,4
;temp = TEMPO_MAX
	movlw	TEMPO_MAX
	movwf	TEMP
;do
SysDoLoop_S7
;delay_200ms
	call	DELAY_200MS
;CMD_GetImage 'envia comando p/ fazer leitura de digital
	call	CMD_GETIMAGE
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;temp = temp - 1
	decf	TEMP,F
;Loop while ((resp <> FP_OK) and  (temp > 0 ))
	movf	resp,W
	movwf	SysBYTETempA
	clrf	SysBYTETempB
	call	SysCompEqual
	comf	SysByteTempX,F
	movf	SysByteTempX,W
	movwf	SysTemp1
	movf	temp,W
	movwf	SysBYTETempB
	clrf	SysBYTETempA
	call	SysCompLessThan
	movf	SysByteTempX,W
	movwf	SysTemp2
	movf	SysTemp1,W
	andwf	SysTemp2,W
	movwf	SysTemp3
	btfsc	SysTemp3,0
	goto	SysDoLoop_S7
SysDoLoop_E7
;if temp = 0 then
	movf	TEMP,F
	btfss	STATUS, Z
	goto	ENDIF18
;pisca_led_erro
	call	PISCA_LED_ERRO
;led_finger = 0
	bcf	PORTB,4
;Exit Sub 'sai da rotina por erro
	return
;end if
ENDIF18
;apaga led finger ('retire o dedo')
;aguarda a retirada do dedo do sensor
;led_finger = 0
	bcf	PORTB,4
;temp = TEMPO_MAX
	movlw	TEMPO_MAX
	movwf	TEMP
;do
SysDoLoop_S8
;delay_200ms
	call	DELAY_200MS
;CMD_GetImage 'envia comando p/ fazer leitura de digital
	call	CMD_GETIMAGE
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;temp = temp - 1
	decf	TEMP,F
;Loop while ((resp <> FP_NOFINGER) and  (temp > 0 ))
	movf	resp,W
	movwf	SysBYTETempA
	movlw	2
	movwf	SysBYTETempB
	call	SysCompEqual
	comf	SysByteTempX,F
	movf	SysByteTempX,W
	movwf	SysTemp1
	movf	temp,W
	movwf	SysBYTETempB
	clrf	SysBYTETempA
	call	SysCompLessThan
	movf	SysByteTempX,W
	movwf	SysTemp2
	movf	SysTemp1,W
	andwf	SysTemp2,W
	movwf	SysTemp3
	btfsc	SysTemp3,0
	goto	SysDoLoop_S8
SysDoLoop_E8
;cria mais um modelo e coloca no buffer 2
;CMD_FP_Image2Tz(2) 'envia comando p/ fazer um modelo da digital
	movlw	2
	movwf	NUMBUF
	call	CMD_FP_IMAGE2TZ
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF19
;pisca_led_erro
	call	PISCA_LED_ERRO
;Exit Sub 'sai da rotina por erro
	return
;end if
ENDIF19
;cria uma fuso de ambos modelos
;CMD_FP_CreateModel
	call	CMD_FP_CREATEMODEL
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if  resp <> FP_OK then
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF20
;pisca_led_erro
	call	PISCA_LED_ERRO
;Exit Sub;}//sai da rotina por erro
	return
;end if
ENDIF20
;armazena na flash
;CMD_StoreModel(pos_gravar);
	movf	POS_GRAVAR,W
	movwf	LOCATION
	call	CMD_STOREMODEL
	movf	LOCATION,W
	movwf	POS_GRAVAR
;resp = Get_AckPack 'recebe dados
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF21
;pisca_led_erro
	call	PISCA_LED_ERRO
;Exit Sub;}//sai da rotina por erro
	return
;end if
ENDIF21
;sucesso! Cadastrou a pessoa e indica piscando 5 vezes o led finger
;for LoopCounter = 1 to 5 'pisca led finger 5 vezes para indicar sucesso
	clrf	LOOPCOUNTER
SysForLoop4
	incf	LOOPCOUNTER,F
;led_finger = 1
	bsf	PORTB,4
;delay_100ms
	call	DELAY_100MS
;led_finger = 0
	bcf	PORTB,4
;delay_100ms
	call	DELAY_100MS
;next
	movlw	5
	subwf	LOOPCOUNTER,W
	btfss	STATUS, C
	goto	SysForLoop4
SysForLoopEnd4
	return

;********************************************************************************

HEADER
;txreg=0xef  'header 2 bytes
	movlw	239
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop8
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop8
;txreg=0x01
	movlw	1
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop9
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop9
;txreg=0xff  'numero do aparelhos 4 bytes (de
	movlw	255
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop10
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop10
;txreg=0xff
	movlw	255
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop11
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop11
;txreg=0xff
	movlw	255
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop12
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop12
;txreg=0xff
	movlw	255
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop13
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop13
;txreg=0x01  'modo comando
	movlw	1
	banksel	TXREG
	movwf	TXREG
;wait until trmt=1 '
SysWaitLoop14
	banksel	TXSTA
	btfss	TXSTA,TRMT
	goto	SysWaitLoop14
	banksel	STATUS
	return

;********************************************************************************

INITSYS
;Set up internal oscillator
;Handle OSCCON1 register for parts that have this register
;Turn off comparator - this is the default setting
;CMCON = 7
	movlw	7
	movwf	CMCON
;Turn off all ports
;PORTA = 0
	clrf	PORTA
;PORTB = 0
	clrf	PORTB
	return

;********************************************************************************

PISCA_LED_ERRO
;CREN=0;//elimina erro de recepo da uart xxxxxxxxxxxxxxxxxfalta
	bcf	RCSTA,CREN
;for LoopCounter = 0 to 4
	movlw	255
	movwf	LOOPCOUNTER
SysForLoop2
	incf	LOOPCOUNTER,F
;led_erro=1
	bsf	PORTA,2
;delay_100ms
	call	DELAY_100MS
;led_erro=0
	bcf	PORTA,2
;delay_100ms
	call	DELAY_100MS
;next
	movlw	4
	subwf	LOOPCOUNTER,W
	btfss	STATUS, C
	goto	SysForLoop2
SysForLoopEnd2
;CREN=1
	bsf	RCSTA,CREN
	return

;*******************************************************************************
;       Procura uma Digital Similar a que foi Escaneada
;*******************************************************************************/
PROCURAR_DIGITAL
;led_finger=1 'acende led finger (coloque o dedo)
	bsf	PORTB,4
;CMD_GetImage
	call	CMD_GETIMAGE
;resp = Get_AckPack
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then Exit Sub;
	movf	RESP,F
	btfss	STATUS, Z
	return
;led_finger = 0 'ok, apaga led finger (pode retirar o dedo)
	bcf	PORTB,4
;CMD_FP_Image2Tz(1) 'faz um modelo de dados para comparao
	movlw	1
	movwf	NUMBUF
	call	CMD_FP_IMAGE2TZ
;resp = Get_AckPack
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then pisca_led_erro : Exit Sub
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF10
	call	PISCA_LED_ERRO
	return
ENDIF10
;CMD_FastSearch 'procura na flash e compara com buffer 1
	call	CMD_FASTSEARCH
;resp = Get_AckPack'
	call	FN_GET_ACKPACK
	movf	GET_ACKPACK,W
	movwf	RESP
;if resp <> FP_OK then pisca_led_erro : Exit Sub
	movf	RESP,F
	btfsc	STATUS, Z
	goto	ENDIF11
	call	PISCA_LED_ERRO
	return
ENDIF11
;if IDlido = 0 then 'grava outras digitais em modo adm
	movf	IDLIDO,F
	btfss	STATUS, Z
	goto	ELSE12_1
;numID = numID+1
	incf	NUMID,F
;if numID >= NUM_MAX_FP then
	movlw	NUM_MAX_FP
	subwf	NUMID,W
	btfss	STATUS, C
	goto	ELSE13_1
;pisca_led_erro
	call	PISCA_LED_ERRO
;numID = numID-1
	decf	NUMID,F
;Exit Sub
	return
;else
	goto	ENDIF13
ELSE13_1
;gravar_digital(numID)
	movf	NUMID,W
	movwf	POS_GRAVAR
	call	GRAVAR_DIGITAL
	movf	POS_GRAVAR,W
	movwf	NUMID
;epread 0, aux
	banksel	EEADDRESS
	clrf	EEADDRESS
	banksel	STATUS
	call	SYSEPREAD
	banksel	EEDATAVALUE
	movf	EEDATAVALUE,W
	banksel	AUX
	movwf	AUX
;if numID <> aux then epwrite(0 , numID)'salva id
	subwf	NUMID,W
	btfsc	STATUS, Z
	goto	ENDIF14
	banksel	EEADDRESS
	clrf	EEADDRESS
	banksel	NUMID
	movf	NUMID,W
	banksel	EEDATAVALUE
	movwf	EEDATAVALUE
	banksel	STATUS
	call	EPWRITE
ENDIF14
;led_prog = 0 'apaga led de programao
	bcf	PORTA,0
;end if
ENDIF13
;else 'encontrou digital cadastrada
	goto	ENDIF12
ELSE12_1
;cren = 0;
	bcf	RCSTA,CREN
;led_acesso_ok = 1 'liga led trava (aciona fechadura eletrica tambm)
	bsf	PORTB,0
;delay_seg(TEMPO_FECHADURA_ACIONADA) 'tempo que fechadura fica energizada
	movlw	TEMPO_FECHADURA_ACIONADA
	movwf	VALOR
	call	DELAY_SEG
;cren = 1;
	bsf	RCSTA,CREN
;led_acesso_ok = 0 'desliga led trava
	bcf	PORTB,0
;end if
ENDIF12
	return

;********************************************************************************

SYSCOMPEQUAL
;Dim SysByteTempA, SysByteTempB, SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;movf SysByteTempA, W
	movf	SYSBYTETEMPA, W
;subwf SysByteTempB, W
	subwf	SYSBYTETEMPB, W
;btfsc STATUS, Z
	btfsc	STATUS, Z
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSCOMPLESSTHAN
;Dim SysByteTempA, SysByteTempB, SysByteTempX as byte
;clrf SysByteTempX
	clrf	SYSBYTETEMPX
;bsf STATUS, C
	bsf	STATUS, C
;movf SysByteTempB, W
	movf	SYSBYTETEMPB, W
;subwf SysByteTempA, W
	subwf	SYSBYTETEMPA, W
;btfss STATUS, C
	btfss	STATUS, C
;comf SysByteTempX,F
	comf	SYSBYTETEMPX,F
	return

;********************************************************************************

SYSEPREAD
;Variable alias
;Dim EEAddress Alias EEADR
;Dim EEDataValue Alias EEDATL_REF
;Disable interrupt
;IntOff
;Select data memory
;Read
;SET RD ON
	banksel	EECON1
	bsf	EECON1,RD
;Restore interrupt
;IntOn
	banksel	STATUS
	return

;********************************************************************************


 END
