/*******************************************************************************


    Fechadura Eletrnica Biomtrica para Acesso a Ambiente Privados
  
    Verso para MPLABX IDE v3.05 Compiler

    c/ PIC16f628a e mdulo sensor biomtrico DY50

    Autor: Claudio Lrios

    Data: 28/05/2020

    Microcontrolador: PIC 16f628a


    Objetivo: Acesso a ambientes privados atravs do uso da impresso digital

    Cadastrar digital do Administrador do Sistema:
    Comece cadastrando a digital da pessoa que ser o administrador.
    Aperte o interruptor 'sw_prog'. O administrador cadastrar o dedo que ser
    usado  para permitir gravar outras digitais no sistema.
    Ao apagar o led finger, coloque de novo o mesmo dedo. Quando piscar este led
    indicar que houve sucesso na gravao.

    Cadastrar outras digitais:
    O administrador usa o dedo cadastrado para iniciar o processo. Assim que co-
    locar o dedo dever acender o led prog. Pea para a outra pessoa colocar o
    dedo que usar para o acesso. Ao apagar o led finger pea que coloque de
    novo o mesmo dedo.
    Quando piscar este led vrias vezes indicar sucesso na gravao.
    Repita os mesmos passos com cada pessoa que ter acesso ao ambiente.

    Deletar todas as digitais:
    Desligue a alimentao do circuito. Aperte o boto sw_del_lib e mantenha
    apertado. Ligue a alimentao. Quando led prog piscar varias vezes, solte
    o boto sw_del_lib. Recadastre novamente as digitais conforme os passos
    acima.

    Ateno: use um adaptador de tenso de 3v3 para 5 volts na sada do sensor
    nomeada de RX ( com 3v no funciona a uart a 57600 bauds). O mesmo para TX.

    Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#define _XTAL_FREQ 20000000//frequncia usada (xtal externo 20mhz))

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC  = HS   // Oscillator Selection bits (HS oscillator: High-speed crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
#pragma config WDTE  = ON    // Watchdog Timer Enable bit (WDT enabled)
#pragma config PWRTE = ON   // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = ON   // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON   // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP   = OFF  // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD   = OFF  // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP    = OFF  // Flash Program Memory Code Protection bit (Code protection off)

//EEPROM INTERNA DO PIC - SITUAO INICIAL 
__EEPROM_DATA(0,0,0,0,0,0,0,0);//inicia ID com zero aps download de firmware

//DEFINIO DE CONSTANTE PARA O SENSOR BIOMETRICO
#define FP_OK 0x00
#define FP_PACKETRECIEVEERR 0x01
#define FP_NOFINGER 0x02
#define FP_IMAGEFAIL 0x03
#define FP_IMAGEMESS 0x06
#define FP_FEATUREFAIL 0x07
#define FP_NOMATCH 0x08
#define FP_NOTFOUND 0x09
#define FP_ENROLLMISMATCH 0x0A
#define FP_BADLOCATION 0x0B
#define FP_DBRANGEFAIL 0x0C
#define FP_UPLOADFEATUREFAIL 0x0D
#define FP_PACKETRESPONSEFAIL 0x0E
#define FP_UPLOADFAIL 0x0F
#define FP_DELETEFAIL 0x10
#define FP_DBCLEARFAIL 0x11
#define FP_PASSFAIL 0x13
#define FP_INVALIDIMAGE 0x15
#define FP_FLASHERR 0x18
#define FP_INVALIDREG 0x1A
#define FP_ADDRCODE 0x20
#define FP_PASSVERIFY 0x21
#define FP_STARTCODE 0xEF01
#define FP_COMMANDPACKET 0x1
#define FP_DATAPACKET 0x2
#define FP_ACKPACKET 0x7
#define FP_ENDDATAPACKET 0x8
#define FP_TIMEOUT 0xFF
#define FP_BADPACKET 0xFE
#define FP_GETIMAGE 0x01
#define FP_IMAGE2TZ 0x02
#define FP_REGMODEL 0x05
#define FP_STORE 0x06
#define FP_LOAD 0x07
#define FP_UPLOAD 0x08
#define FP_DELETE 0x0C
#define FP_EMPTY 0x0D
#define FP_SETPASSWORD 0x12
#define FP_VERIFYPASSWORD 0x13
#define FP_HISPEEDSEARCH 0x1B
#define FP_TEMPLATECOUNT 0x1D

//DEFINIO DE PINOS
#define    sw_del_lib     RB5 // 0x6.5  //pino 11 -deletar todas as digitais
#define    sw_prog        RA4 // 0x5.4 //pin 3 - programar digital do ADMINISTRADOR
#define    led_finger     RB4 // 0x6.4 //pino 10 - indica para colocar/retirar o dedo
#define    led_acesso_ok  RB0 // 0x6.0 //pino 6  - tambm sada para acionar fechadura
#define    led_prog       RA0 // 0x5.0 //pino 17 - aceso= modo prog/ piscando=apagamento
#define    led_erro       RA2 // 0x5.2 //pino 1 - led indicador de algum erro na operao
#define    led_com_ok     RA1 // 0x5.1 //pino 18 -led programao realizado com sucesso
#define    led_teste      RA3 // 0x5.3 //pino 2
//DEFINIO DE CONSTANTES
#define NUM_MAX_FP 163  //Numero mximo de digitais que podem ser cadastradas
#define TEMPO_FECHADURA_ACIONADA 2 // em segundos
#define TEMPO_MAX 180 //tempo mximo em modo gravao (aprox. 40 segundos)


//PROTTIPOS DAS FUNOES
void Header();
void CMD_Verify_PassWord();
void CMD_GetImage();
void CMD_FP_Image2Tz(unsigned char ID);
void CMD_FP_CreateModel();
void CMD_StoreModel(unsigned char location);
void pisca_led_erro();
void gravar_digital(unsigned char id);
unsigned char Get_AckPack();
void delay_seg(unsigned char v);
void delay_200ms();
void delay_100ms();
void delay_200us();
void del_all_flash();

//VARIVEIS GLOBAIS
unsigned char  data[13];//buffer de recepo do pacote de dados via uart
unsigned char  p;//uso geral
unsigned char  a;//uso geral
unsigned char  ID_lido;//retorna endereo que est a digital encontrada
unsigned char  numID=0;//posio que ser gravada o template na flash


/*==============================================================================
                      Rotinas de Delay
==============================================================================*/

void delay_seg(unsigned char v){//rotina de delay ajustavel, em segundos
  unsigned char a;
  for(a=0;a<v;a++){
      CLRWDT();//zera wtd
    __delay_ms(1000);
  }
}

void delay_200ms(){// 200 milissegundos
     CLRWDT();//zera wtd
   __delay_ms(200);
}
void delay_100ms(){// 100 milissegundos
     CLRWDT();//zera wtd
   __delay_ms(100);
}

void delay_200us(){// 200 microssegundos
    CLRWDT();//zera wtd
   __delay_us(200);
}

/*******************************************************************************
      Rotina  Indicadora de Erro Encontrado em algum Processo Realizado
*******************************************************************************/

void pisca_led_erro(){
 unsigned char  c;

 CREN=0;//elimina erro de recepo da uart
 for(c=0;c<4;c++){
   led_erro=1;
   delay_100ms();
   led_erro=0;
   delay_100ms();
  }
 CREN=1;
}

/*******************************************************************************
      Rotina de Envio do Header (fixado em modo 'comando')
      0xEF01= Inicio
      0xFFFFFFFF= Endereo Default
      0x01 == modo envio de comando
   Obs. Para usar outro endero necessitar alterar a rotina abaixo.
*******************************************************************************/
void Header(){

   TXREG=0xef;//header 2 bytes
   while(!TRMT);//
   TXREG=0x01;
   while(!TRMT);

   TXREG=0xff;//numero do aparelhos 4 bytes (de
   while(!TRMT);
   TXREG=0xff;
   while(!TRMT);
   TXREG=0xff;
   while(!TRMT);
   TXREG=0xff;
   while(!TRMT);

   TXREG=0x01;//modo comando
   while(!TRMT);

}
/*******************************************************************************
      Rotina de Envio da Instruo 'CMD_VerifyPassWord'
      0x00000000 = envia 'Password Default'
  obs. Para usar outra 'password' necessitar alterar a rotina abaixo.
*******************************************************************************/
void CMD_Verify_PassWord(){
 unsigned char  a;
   Header();//envia header em modo comando

   TXREG=0x00;//lenght 2 bytes
   while(!TRMT);
   TXREG=0x07;
   while(!TRMT);

   TXREG=FP_VERIFYPASSWORD;//comando 0x13
   while(!TRMT);
   for(a=0; a<5;a++){//envia senha 0x00000000 e msb sum=0x00
     TXREG=0x00;//senha 4 bytes
    while(!TRMT);
   }
    TXREG=0x1b;//sum lsb
   while(!TRMT);
}


/*******************************************************************************
      Rotina  de obteno da digital - CMD_Get_Image
*******************************************************************************/
void CMD_GetImage(){

   Header();//envia header em modo comando

   TXREG=0x00;//lenght msb
   while(!TRMT);
   TXREG=0x03;//lenght lsb
   while(!TRMT);

   TXREG=FP_GETIMAGE;//0x01;//comando
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);
    TXREG=0x05;//sum lsb
   while(!TRMT);
}

/*******************************************************************************
      Rotina  de Gerao de Arquivo e Armazenamento em Buffer
      ID= 1 ou 2 (apenas 2 buffers)
*******************************************************************************/
void CMD_FP_Image2Tz(unsigned char numbuf){
   if(numbuf!=1)numbuf=2;
   Header();//envia header em modo comando

   TXREG=0x00;//lenght msb
   while(!TRMT);
   TXREG=0x04; //lenght lsb
   while(!TRMT);

   TXREG=FP_IMAGE2TZ;//0x02;//comando
   while(!TRMT);

   TXREG=numbuf;
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);

   if(numbuf==1){//ID=1
     TXREG=0x08;//sum lsb
   }
   else{//ID=2
    TXREG=0x09;//sum lsb
   }
   while(!TRMT);
}



/*******************************************************************************
      Rotina  de Criao de Modlo e Armazenamento em Buffer
      ID= 1 ou 2 (apenas 2 buffers)
*******************************************************************************/
void CMD_FP_CreateModel(){

   Header();//envia header em modo comando

   TXREG=0x00;//lenght 2 bytes
   while(!TRMT);
   TXREG=0x03;
   while(!TRMT);

   TXREG=FP_REGMODEL;//0x05;//comando
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);
   TXREG=0x09;//sum lsb
   while(!TRMT);
}

/*******************************************************************************
      Rotina  de Armazenamento de Arquivo na Memria Flash
      ID= 1 ou 2 (apenas 2 buffers)
*******************************************************************************/
void CMD_StoreModel(unsigned char location){
 unsigned char  sum=14;

   sum+=location;//soma ao sum
   Header();//envia header em modo comando

   TXREG=0x00;//lenght msb
   while(!TRMT);
   TXREG=0x06;//lenght lsb
   while(!TRMT);

   TXREG=FP_STORE;//0x06;//comando
   while(!TRMT);

   TXREG=0x01;// aponta para  o modelo no buffer 1
   while(!TRMT);

   TXREG=0x00;//id msb (max 160 imagens)
   while(!TRMT);

   TXREG=location;//id lsb
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);
   TXREG=sum;//sum lsb
   while(!TRMT);
}

/*******************************************************************************
      Rotina  de Pesquisa por digital igual a uma j gravada na  Memria Flash
      obs. Fixado no Buffer 1 e nas paginas 0x0 ate 0xA3
*******************************************************************************/

 void CMD_FastSearch(){

   Header();//envia header em modo comando

   TXREG=0x00;//lenght msb
   while(!TRMT);
   TXREG=0x08;//lenght lsb
   while(!TRMT);

   TXREG=FP_HISPEEDSEARCH;// 0x1B comando
   while(!TRMT);

   TXREG=0x01;//buffer ID escolhido para comparao
   while(!TRMT);

   TXREG=0x00;//pag inicial msb
   while(!TRMT);
   TXREG=0x00;//pag inicial lsb
   while(!TRMT);

   TXREG=0x00;//pag final msb
   while(!TRMT);
   TXREG=0xA3;//pag final lsb
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);
   TXREG=0xc8;//sum lsb
   while(!TRMT);
}

/*******************************************************************************
      Rotina  para Deletar todas as Digitais Armazenadas na Flash
*******************************************************************************/

 void CMD_FP_Empty(){

   Header();//envia header em modo comando

   TXREG=0x00;//lenght msb
   while(!TRMT);
   TXREG=0x03;//lenght lsb
   while(!TRMT);

   TXREG=FP_EMPTY;// 0x0d comando
   while(!TRMT);

   TXREG=0x00;//sum msb
   while(!TRMT);
   TXREG=0x11;//sum lsb
   while(!TRMT);
}

/*******************************************************************************
       Procura uma Digital Similar a que foi Escaneada
*******************************************************************************/
void procurar_digital(){
   led_finger=1;//acende led finger (coloque o dedo)
   CMD_GetImage();
   p=Get_AckPack();
   if(p!=FP_OK)return;
   led_finger=0;//ok, apaga led finger (pode retirar o dedo)


   CMD_FP_Image2Tz(1);//faz um modelo de dados para comparao
   p=Get_AckPack();
   if(p!=FP_OK){ pisca_led_erro(); return;}

   CMD_FastSearch();//procura na flash e compara com buffer 1
   p=Get_AckPack();
   if(p!=FP_OK){ pisca_led_erro(); return;}

   if(ID_lido==0){//grava outras digitais em modo adm
      numID++;
      if (numID>=NUM_MAX_FP){ pisca_led_erro();numID--; return;}
      else{
        gravar_digital(numID);
        if(numID!=EEPROM_READ(0))EEPROM_WRITE(0,numID);//salva id
        led_prog=0;//apaga led de programao
      }
   }
   else{ //encontrou digital cadastrada
     CREN=0;
     led_acesso_ok=1;  //liga led trava (aciona fechadura eletrica tambm)
     delay_seg(TEMPO_FECHADURA_ACIONADA);//tempo que fechadura fica energizada
     CREN=1;
     led_acesso_ok=0;//desliga led trava
   }
}

 /*******************************************************************************
      Rotina  de Recepo dos dados do Sensor de digital (Acknowledge Packet)
*******************************************************************************/

unsigned char Get_AckPack(){

//ateno: use um adaptador de tenso de 3v3 para 5 volts na sada do sensor
//nomeada de RX ( com 3v no funciona a uart a 57600 bauds).
  unsigned char  a;
   CREN=0;//apaga erros de recepo da uart
   CREN=1;
   a=RCREG;//limpa buffer da usart caso haja algum byte na FIFO
    a=0;

     led_com_ok=0;
    while(a<12){ //recebe 12 bytes
      while(!RCIF);
        data[a]=RCREG; //pega dado recebido
        a++;
        led_com_ok=1;//acende apenas na fase de recebimento de dados
    }//while(a<12)
    led_com_ok=0;
    ID_lido=data[11];
    delay_200us();

    return data[9];
}
/*******************************************************************************
      Rotina de Gravao de uma Digital  na Flash
*******************************************************************************/


void gravar_digital(unsigned char pos_gravar){
unsigned char  p;
unsigned char  t;//nmero de tentativas

   CREN=0;
   led_prog=1;//acende led de programao
   led_finger=1;//acende led 'coloque o dedo'
   CREN=1;
   //1 captura da digital

   t=TEMPO_MAX;
   do{
     delay_200ms();
     CMD_GetImage();//envia comando p/ fazer leitura de digital
     p=Get_AckPack();//recebe dados
     t--;
   }while ((p != FP_OK)&& t);

   if(!t){pisca_led_erro(); led_finger=0; return;}//sai da rotina por erro

   //apaga led finger ('retire o dedo')
   //aguarda a retirada do dedo
   led_finger=0;
   t=TEMPO_MAX;
   do{
     delay_200ms(); //delay
     CMD_GetImage();//envia comando p/ fazer leitura de digital
     p=Get_AckPack();//recebe dados
     t--;
   }while (p != FP_NOFINGER && t);

   if(!t){pisca_led_erro(); return;}//sai da rotina por erro

   //cria um modelo e coloca no buffer 1

  CMD_FP_Image2Tz(1);//envia comando p/ fazer um modelo da digital
  p=Get_AckPack();//recebe dados
  if(p!= FP_OK){pisca_led_erro(); return;}//sai da rotina por erro


  //2 captura da mesma digital
  led_finger=1;//acende led 'coloque o dedo'

   t=TEMPO_MAX;
   do{
     delay_200ms();
     CMD_GetImage();//envia comando p/ fazer leitura de digital
     p=Get_AckPack();//recebe dados
     t--;
   }while ((p != FP_OK)&& t);

   if(!t){pisca_led_erro(); led_finger=0; return;}//sai da rotina por erro


   //apaga led finger ('retire o dedo')
   //aguarda a retirada do dedo do sensor
   led_finger=0;
   t=TEMPO_MAX;
   do{
     delay_200ms();
     CMD_GetImage();//envia comando p/ fazer leitura de digital
     p=Get_AckPack();//recebe dados
     t--;
   }while (p != FP_NOFINGER && t);



 //cria mais um modelo e coloca no buffer 2
  CMD_FP_Image2Tz(2);//envia comando p/ fazer um modelo da digital
  p=Get_AckPack();//recebe dados
  if(p!= FP_OK){pisca_led_erro(); return;}//sai da rotina por erro


   //cria uma fuso de ambos modelos

   CMD_FP_CreateModel();
   p=Get_AckPack();//recebe dados
   if (p != FP_OK){pisca_led_erro(); return;}//sai da rotina por erro

   //armazena na flash
   CMD_StoreModel(pos_gravar);
   p=Get_AckPack();//recebe dados
   if (p != FP_OK){pisca_led_erro();  return;}//sai da rotina por erro

   //sucesso! Libera Acesso para  a pessoa acionando fechadura
   for(p=0;p<5;p++){//pisca led finger 5 vezes para indicar sucesso
   led_finger=1;
   delay_100ms();
   led_finger=0;
   delay_100ms();
   }


}
/*******************************************************************************
      Rotina de Apagamento de todas as digitais  na Flash
      Obs. Deve ser executado somente no setup (apertando boto sw_del_lib
      e religando a alimentao)
*******************************************************************************/
void del_all_flash(){
   if(!sw_del_lib){//testa boto de apagamento de todas as digitais gravadas
    led_com_ok=1;
    delay_seg(5);
    if(!sw_del_lib){
      CMD_FP_Empty();
       p=Get_AckPack();//obtm a resposta
        if(p!=FP_OK) {
          pisca_led_erro();//erro no apagamento geral. No houve apagamento!
        }
        else{
   //se apagamento geral foi bem sucedido pisca_led prog 10 vezes
          numID=0;
          EEPROM_WRITE(0,numID);//zera ponteiro
          for(a=0;a<10;a++){
            led_prog=1;
            delay_100ms();
            led_prog=0;
            delay_100ms();
          }
        }

  }
   led_com_ok=0;
   while(!sw_del_lib)delay_100ms();
  }

}
/*==============================================================================

                      ROTINA MAIN

==============================================================================*/



int main(int argc, char** argv) {

   OPTION_REG=0X8F;//wdt no mximo de tempo
   CLRWDT();//zera wtd

   CMCON=7;//desliga comparadores internos
   TRISA=0b11110000;
   TRISB=0b11101110;//TRISB 2:1 = 1 para usart operar
   SPBRG=21;//xtal 20mhz 57600 bauds  BRGH=1
   RCSTA= 0b10010000; //assyncronous rx config pino 7
   TXSTA= 0b10100100; //assyncronous tx config  pino 8
   PORTB=0;
   PORTA=0;
   numID=EEPROM_READ(0);//retorna id
   led_com_ok=1;
   delay_seg(1); //boot-up do sensor
   led_com_ok=0;

   CMD_Verify_PassWord();//envia comando senha do sensor
   p=Get_AckPack();//obtm a resposta  //erro no funciona a recepo


   if(p!=FP_OK) {
       pisca_led_erro();//algum erro ocorreu, necessita reset
       for(;;);//aguarda wtd timer fazer reset (ou manual)
   }


   del_all_flash();//testa se deve deletar toda as impresses digitais da flash


/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/


 for(;;){
  CLRWDT();//zera wtd//zera wtd

//BOTO DE GRAVAR ADM
  if(!sw_prog){
   led_prog=1;//acende led de programao
   while(!sw_prog)delay_100ms();//aguarda soltar boto sw_prog
  }
  if(led_prog){
    gravar_digital(0);//grava digital do administrador
    led_prog=0;//sai do modo de programao da digital do adm
  }

  procurar_digital(); //verifica se existe digital a ser comparada
  
 }// for(;;)
   
  return (EXIT_SUCCESS);
}//main


/*--------------------fim do programa-----------------------------------------*/
